@@ -2,6 +2,7 @@
 #include "port.h"
 #include "DnaPSSM.h"
 #include "Random.h"
+#include <R_ext/Arith.h> 
 
 #include <algorithm>
 
@@ -18,22 +19,22 @@ void DnaProbVec::normalize()
 	if(m_p[0] != 0) {
 		m_logp[0] = log(m_p[0]);
 	} else {
-		m_logp[0] = -_REAL(MAX)/100;
+		m_logp[0] = R_NegInf;
 	}
 	if(m_p[1] != 0) {
 		m_logp[1] = log(m_p[1]);
 	} else {
-		m_logp[1] = -_REAL(MAX)/100;
+		m_logp[1] = R_NegInf;
 	}
 	if(m_p[2] != 0) {
 		m_logp[2] = log(m_p[2]);
 	} else {
-		m_logp[2] = -_REAL(MAX)/100;
+		m_logp[2] = R_NegInf;
 	}
 	if(m_p[3] != 0) {
 		m_logp[3] = log(m_p[3]);
 	} else {
-		m_logp[3] = -_REAL(MAX)/100;
+		m_logp[3] = R_NegInf;
 	}
 }
 void DnaProbVec::normalize_log()
@@ -120,9 +121,13 @@ void DnaPSSM::init_from_seed(const string &seed, float prior)
 	for(string::const_iterator i = seed.begin(); i != seed.end(); i++) {
 		p->reset(back);
 		switch(*i) {
+			case 'a':
 			case 'A': p->set_direct_prob(0, 1 - 3*prior); break;
+			case 'c':
 			case 'C': p->set_direct_prob(1, 1 - 3*prior); break;
+			case 'g':
 			case 'G': p->set_direct_prob(2, 1 - 3*prior); break;
+			case 't':
 			case 'T': p->set_direct_prob(3, 1 - 3*prior); break;
 		}
 		p->normalize();
@@ -150,8 +155,11 @@ void DnaPSSM::calc_like(const string &target, float &logp) const
 	for(vector<DnaProbVec>::const_iterator p = m_chars.begin();
 	    p < m_chars.end();
 	    p++) {
-		if(*i != 'A' && *i != 'C' && *i != 'G' && *i != 'T') {
-			logp = -_REAL(MAX);
+		if(
+			*i != 'A' && *i != 'C' && *i != 'G' && *i != 'T' && 
+			*i != 'a' && *i != 'c' && *i != 'g' && *i != 't'
+			) {
+			logp = R_NegInf;
 			return;
 		}
 		logp += p->get_log_prob(*i);
@@ -168,15 +176,19 @@ void DnaPSSM::calc_like_rc(const string &target, float &logp) const
 	    p++) {		
 		char c;
 		switch(*i) {
+			case 'a':
 			case 'A': c = 'T';
 				  break;
+			case 't':
 			case 'T': c = 'A';
 				  break;
+			case 'c':
 			case 'C': c = 'G';
 				  break;
+			case 'g':
 			case 'G': c = 'C';
 				  break;
-			default:  logp = -_REAL(MAX);
+			default:  logp = R_NegInf;
 				  return;
 		}
 		logp += p->get_log_prob(c);
@@ -191,8 +203,11 @@ void DnaPSSM::calc_like(string::const_iterator &j, float &logp) const
 	for(vector<DnaProbVec>::const_iterator p = m_chars.begin();
 	    p < m_chars.end();
 	    p++) {
-		if(*i != 'A' && *i != 'C' && *i != 'G' && *i != 'T') {
-			logp = -_REAL(MAX);
+		if(
+			*i != 'A' && *i != 'C' && *i != 'G' && *i != 'T' && 
+			*i != 'a' && *i != 'c' && *i != 'g' && *i != 't'
+			) {
+			logp = R_NegInf;
 			return;
 		}
 		logp += p->get_log_prob(*i);
@@ -212,15 +227,19 @@ void DnaPSSM::calc_like_rc(string::const_iterator &j, float &logp) const
 	    p++) {
 		char c;
 		switch(*i) {
+			case 'a':
 			case 'A': c = 'T';
 				  break;
+			case 't':
 			case 'T': c = 'A';
 				  break;
+			case 'c':
 			case 'C': c = 'G';
 				  break;
+			case 'g':
 			case 'G': c = 'C';
 				  break;
-			default:  logp = -_REAL(MAX);
+			default:  logp = R_NegInf;
 				  return;
 		}
 		logp += p->get_log_prob(c);
@@ -229,10 +248,10 @@ void DnaPSSM::calc_like_rc(string::const_iterator &j, float &logp) const
 }
 
 string::const_iterator DnaPSSM::max_like_match(const string &target,
-				float &best_logp, int &best_dir) const
+				float &best_logp, int &best_dir, const bool &combine_strands) const
 {
 	if(target.length() < m_chars.size()) {
-		best_logp = -_REAL(MAX);
+		best_logp = R_NegInf;
 		return(target.begin());
 	}
 
@@ -241,9 +260,9 @@ string::const_iterator DnaPSSM::max_like_match(const string &target,
 		max_i = target.end() - m_chars.size();
 	}
 	string::const_iterator best_pos;
-	best_logp = -_REAL(MAX)/100;
+	best_logp = R_NegInf;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		string::const_iterator j = i;
 		float logp = 0;
@@ -251,62 +270,79 @@ string::const_iterator DnaPSSM::max_like_match(const string &target,
 		    p < m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
-			if(*j == 'N' || *j =='*') {
+			if(*j == 'N' || *j =='*' || *j == 'n') {
 				logp += p->get_avg_log_prob();
 			} else {
 				logp += p->get_log_prob(*j);
-			}
-			if(logp < best_logp) {
-				break;
-			}
+			}			
 			j++;
 		}
-		if(logp > best_logp) {
-			best_logp = logp;
-			best_pos = i;
-			best_dir = 1;
-		}
+
+		float total_logp = logp;
+
 		if(m_bidirect) {
-			logp = 0;
+			float rlogp = 0;
 			j = i;
 			for(vector<DnaProbVec>::const_reverse_iterator p = m_chars.rbegin();
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					logp = -_REAL(MAX);
+					rlogp = R_NegInf;
 					break;
 				}
 				
 				switch(*j) {
-					case 'A': logp += p->get_log_prob('T');
+					case 'a':
+					case 'A': rlogp += p->get_log_prob('T');
 						  break;
-					case 'T': logp += p->get_log_prob('A');
+					case 't':
+					case 'T': rlogp += p->get_log_prob('A');
 						  break;
-					case 'C': logp += p->get_log_prob('G');
+					case 'c':
+					case 'C': rlogp += p->get_log_prob('G');
 						  break;
-					case 'G': logp += p->get_log_prob('C');
+					case 'g':
+					case 'G': rlogp += p->get_log_prob('C');
 						  break;
-					case '*': logp += p->get_avg_log_prob();
+					case '*': rlogp += p->get_avg_log_prob();
 						  break;
-					case 'N': logp += p->get_avg_log_prob();
+					case 'n':
+					case 'N': rlogp += p->get_avg_log_prob();
 						  break;
 					default:  break;
 				}
 				j++;
 			}
-			if(logp > best_logp) {
-				best_logp = logp;
-				best_pos = i;
-				best_dir = -1;
+
+			if (combine_strands) {
+				// Combine probabilities by adding them in log space
+				log_sum_log(total_logp, rlogp);
+				best_dir = 0; // Indicate combined strands
+			} else {
+				// select best strand
+				if(rlogp > logp) {
+					total_logp = rlogp;
+					best_dir = -1;
+				} else {
+					best_dir = 1;
+				}
 			}
+		} else {
+			best_dir = 1;
+		}
+
+		if(total_logp > best_logp) {
+			best_logp = total_logp;
+			best_pos = i;
 		}
 	}
 	return(best_pos);
 }
 
+
 void DnaPSSM::update_like_vec(const string &target,
 			vector<float> &likes, vector<float> &deltas,
 			vector<int1> &dirs)
@@ -323,7 +359,7 @@ void DnaPSSM::update_like_vec(const string &target,
 	vector<float>::iterator like = likes.begin() + m_min_range;
 	vector<int1>::iterator dir = dirs.begin() + m_min_range;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		string::const_iterator j = i;
 		float logp = 0;
@@ -331,10 +367,10 @@ void DnaPSSM::update_like_vec(const string &target,
 		    p < m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
-			if(*j == 'N' || *j =='*') {
+			if(*j == 'N' || *j =='*' || *j == 'n') {
 				logp += c_log_quarter;
 			} else {
 				logp += p->get_log_prob(*j);
@@ -350,21 +386,26 @@ void DnaPSSM::update_like_vec(const string &target,
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					rlogp = -_REAL(MAX);
+					rlogp = R_NegInf;
 					break;
 				}
 				
 				switch(*j) {
+					case 'a':
 					case 'A': rlogp += p->get_log_prob('T');
 						  break;
+					case 't': 
 					case 'T': rlogp += p->get_log_prob('A');
 						  break;
+					case 'c':
 					case 'C': rlogp += p->get_log_prob('G');
 						  break;
+					case 'g':
 					case 'G': rlogp += p->get_log_prob('C');
 						  break;
 					case '*': rlogp += p->get_avg_log_prob();
 						  break;
+					case 'n':
 					case 'N': rlogp += p->get_avg_log_prob();
 						  break;
 					default:  break;
@@ -376,9 +417,9 @@ void DnaPSSM::update_like_vec(const string &target,
 				*dir = -1;
 			}
 		}
-		if(logp == -_REAL(MAX)) {
-			*delta = -_REAL(MAX);
-			*like = -_REAL(MAX);
+		if(logp == R_NegInf) {
+			*delta = R_NegInf;
+			*like = R_NegInf;
 		} else {
 			*delta = -(*like);
 			*delta += logp;
@@ -392,9 +433,9 @@ void DnaPSSM::update_like_vec(const string &target,
 
 void DnaPSSM::integrate_like_seg(const char *min_i, const char *max_i, float &energy) const
 {
-	energy = -_REAL(MAX)/100;
+	energy = R_NegInf;
 	for(const char *i = min_i;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		const char *j = i;
 		float logp = 0;
@@ -402,7 +443,7 @@ void DnaPSSM::integrate_like_seg(const char *min_i, const char *max_i, float &en
 		    p < m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
 			if(*j == 'N' || *j == 'n' || *j =='*') {
@@ -424,7 +465,7 @@ void DnaPSSM::integrate_like_seg(const char *min_i, const char *max_i, float &en
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					logp = -_REAL(MAX);
+					logp = R_NegInf;
 					break;
 				}
 				
@@ -463,7 +504,7 @@ void DnaPSSM::integrate_like_seg(const char *min_i, const char *max_i, float &en
 void DnaPSSM::integrate_like(const string &target, float &energy, vector<float> *spat_dist) const
 {
 	if(target.length() < m_chars.size()) {
-		energy = -_REAL(MAX);
+		energy = R_NegInf;
 		return;
 	}
 
@@ -471,9 +512,9 @@ void DnaPSSM::integrate_like(const string &target, float &energy, vector<float>
 	if(max_i > target.end() - m_chars.size()) {
 		max_i = target.end() - m_chars.size();
 	}
-	energy = -_REAL(MAX)/100;
+	energy = R_NegInf;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		string::const_iterator j = i;
 		float logp = 0;
@@ -481,10 +522,10 @@ void DnaPSSM::integrate_like(const string &target, float &energy, vector<float>
 		    p < m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
-			if(*j == 'N' || *j =='*') {
+			if(*j == 'N' || *j =='*' || *j == 'n') {
 				logp += c_log_quarter;
 			} else {
 				logp += p->get_log_prob(*j);
@@ -503,21 +544,26 @@ void DnaPSSM::integrate_like(const string &target, float &energy, vector<float>
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					logp = -_REAL(MAX);
+					logp = R_NegInf;
 					break;
 				}
 				
 				switch(*j) {
+					case 'a':
 					case 'A': logp += p->get_log_prob('T');
 						  break;
+					case 't':
 					case 'T': logp += p->get_log_prob('A');
 						  break;
+					case 'c':
 					case 'C': logp += p->get_log_prob('G');
 						  break;
+					case 'g':
 					case 'G': logp += p->get_log_prob('C');
 						  break;
 					case '*': logp += p->get_avg_log_prob();
 						  break;
+					case 'n':
 					case 'N': logp += p->get_avg_log_prob();
 						  break;
 					default:  break;
@@ -547,12 +593,16 @@ void DnaPSSM::count(string::const_iterator seq, float weight, int dir)
 		    i++) {
 			char c = 'N';
 			switch(*seq) {
+				case 'a':
 				case 'A': c = 'T';
 					  break;
+				case 't':
 				case 'T': c = 'A';
 					  break;
+				case 'c':
 				case 'C': c = 'G';
 					  break;
+				case 'g':
 				case 'G': c = 'C';
 					  break;
 				default: break;
@@ -578,7 +628,7 @@ void DnaPSSM::count_weighted(const string &target, vector<float> &wgts,
 	vector<float>::iterator wgt = wgts.begin() + m_min_range;
 	vector<int1>::iterator dir = dirs.begin() + m_min_range;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		//if logp is very small - ignore it
 		if(*wgt < thresh_wgt) {
@@ -591,7 +641,7 @@ void DnaPSSM::count_weighted(const string &target, vector<float> &wgts,
 			for(vector<DnaProbVec>::iterator p = m_chars.begin();
 			    p < m_chars.end();
 			    p++) {
-				if((*j) && *j != 'N' && *j !='*') {
+				if((*j) && *j != 'N' && *j !='*' && *j != 'n') {
 					p->incr_weight(*j, *wgt);
 				}
 				j++;
@@ -602,12 +652,16 @@ void DnaPSSM::count_weighted(const string &target, vector<float> &wgts,
 			    p != m_chars.rend();
 			    p++) {
 				switch(*j) {
+					case 'a':
 					case 'A': p->direct_incr_weight(3, *wgt);
 						  break;
+					case 't':
 					case 'T': p->direct_incr_weight(0, *wgt);
 						  break;
+					case 'c':
 					case 'C': p->direct_incr_weight(2, *wgt);
 						  break;
+					case 'g':
 					case 'G': p->direct_incr_weight(1, *wgt);
 						  break;
 					default:  break;
@@ -634,7 +688,7 @@ void DnaPSSM::count_log_weighted(const string &target, vector<float> &wgts,
 	vector<float>::iterator wgt = wgts.begin() + m_min_range;
 	vector<int1>::iterator dir = dirs.begin() + m_min_range;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		//if logp is very small - ignore it
 		if(*wgt < thresh_wgt) {
@@ -647,7 +701,7 @@ void DnaPSSM::count_log_weighted(const string &target, vector<float> &wgts,
 			for(vector<DnaProbVec>::iterator p = m_chars.begin();
 			    p < m_chars.end();
 			    p++) {
-				if((*j) && *j != 'N' && *j !='*') {
+				if((*j) && *j != 'N' && *j !='*' && *j != 'n') {
 					p->incr_log_weight(*j, *wgt);
 				}
 				j++;
@@ -658,12 +712,16 @@ void DnaPSSM::count_log_weighted(const string &target, vector<float> &wgts,
 			    p != m_chars.rend();
 			    p++) {
 				switch(*j) {
+					case 'a':
 					case 'A': p->direct_incr_log_weight(3, *wgt);
 						  break;
+					case 't':
 					case 'T': p->direct_incr_log_weight(0, *wgt);
 						  break;
+					case 'c':
 					case 'C': p->direct_incr_log_weight(2, *wgt);
 						  break;
+					case 'g':
 					case 'G': p->direct_incr_log_weight(1, *wgt);
 						  break;
 					default:  break;
@@ -838,7 +896,7 @@ ostream &operator<<(ostream &out, const DnaPSSM &pssm)
 void DnaPSSM::integrate_energy(const string &target, float &energy, vector<float> &spat_func, int spat_bin_size) const
 {
 	if(target.length() < m_chars.size()) {
-		energy = -_REAL(MAX);
+		energy = R_NegInf;
 		return;
 	}
 
@@ -846,10 +904,10 @@ void DnaPSSM::integrate_energy(const string &target, float &energy, vector<float
 	if(max_i > target.end() - m_chars.size()) {
 		max_i = target.end() - m_chars.size();
 	}
-	energy = -_REAL(MAX)/100;
+	energy = R_NegInf;
 	int pos = 0;
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		int spat_bin = int(pos/spat_bin_size);
 		pos++;
@@ -859,10 +917,10 @@ void DnaPSSM::integrate_energy(const string &target, float &energy, vector<float
 		    p < m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
-			if(*j == 'N' || *j =='*') {
+			if(*j == 'N' || *j =='*' || *j == 'n') {
 				logp += p->get_avg_log_prob();
 			} else {
 				logp += p->get_log_prob(*j);
@@ -879,21 +937,26 @@ void DnaPSSM::integrate_energy(const string &target, float &energy, vector<float
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					logp = -_REAL(MAX);
+					logp = R_NegInf;
 					break;
 				}
 
 				switch(*j) {
+					case 'a':
 					case 'A': logp += p->get_log_prob('T');
 						  break;
+					case 't':
 					case 'T': logp += p->get_log_prob('A');
 						  break;
+					case 'c':
 					case 'C': logp += p->get_log_prob('G');
 						  break;
+					case 'h':
 					case 'G': logp += p->get_log_prob('C');
 						  break;
 					case '*': logp += c_log_quarter;
 						  break;
+					case 'n':
 					case 'N': logp += c_log_quarter;
 						  break;
 					default:  break;
@@ -917,7 +980,7 @@ void DnaPSSM::like_thresh_match(const string &target, float thresh,
 		max_i = target.end() - m_chars.size();
 	}
 	for(string::const_iterator i = target.begin() + m_min_range;
-	    i < max_i;
+	    i <= max_i;
 	    i++) {
 		string::const_iterator j = i;
 		float logp = 0;
@@ -925,10 +988,10 @@ void DnaPSSM::like_thresh_match(const string &target, float thresh,
 		    p != m_chars.end();
 		    p++) {
 			if(!(*j)) {
-				logp = -_REAL(MAX);
+				logp = R_NegInf;
 				break;
 			}
-			if(*j == 'N' || *j =='*') {
+			if(*j == 'N' || *j =='*' || *j == 'n') {
 				logp += c_log_quarter;
 			} else {
 				logp += p->get_log_prob(*j);
@@ -950,21 +1013,26 @@ void DnaPSSM::like_thresh_match(const string &target, float thresh,
 			    p != m_chars.rend();
 			    p++) {
 				if(!(*j)) {
-					logp = -_REAL(MAX);
+					logp = R_NegInf;
 					break;
 				}
 
 				switch(*j) {
+					case 'a':
 					case 'A': logp += p->get_log_prob('T');
 						  break;
+					case 't':
 					case 'T': logp += p->get_log_prob('A');
 						  break;
+					case 'c':
 					case 'C': logp += p->get_log_prob('G');
 						  break;
+					case 'g':
 					case 'G': logp += p->get_log_prob('C');
 						  break;
 					case '*': logp += c_log_quarter;
 						  break;
+					case 'n':
 					case 'N': logp += c_log_quarter;
 						  break;
 					default:  break;
