@@ -1,5 +1,6 @@
 #ifndef _GINTERVALSFETCHER_H_INCLUDED_
 #define _GINTERVALSFETCHER_H_INCLUDED_
+#include <cstdint>
 
 template <typename Interval>
 class GIntervalsFetcher {
@@ -21,8 +22,21 @@ public:
 	virtual void seal() = 0;
 
 	// returns number of intervals in the intervals set
-	virtual size_t size() const = 0;
+	virtual uint64_t size() const = 0;
 
+	// Iterators: there are two modes of iterators
+	// 1. If iterator is started with begin_iter():
+	//    next()          - advances the iterator until end of all chromosomes is reached
+	//    next_in_chrom() - does nothing and returns always false
+	//    isend()         - returns true if end of all chromosomes is reached
+	//    isend_chrom()   - always returns true
+	//    cur_interval()  - returns a valid object as long as isend() returns false
+	// 2. If iterator is started with begin_chrom_iter():
+	//    next()          - advances the iterator until end of all chromosomes is reached
+	//    next_in_chrom() - advances the iterator until end of current chromosome is reached
+	//    isend()         - returns true if end of all chromosomes is reached
+	//    isend_chrom()   - returns true if end of current chromosome is reached
+	//    cur_interval()  - returns a valid object as long as isend_chrom() (and not isend()!) returns false
 	virtual void begin_iter() = 0;
 
 	// returns false if end reached (i.e. isend() returns true)
@@ -36,10 +50,10 @@ public:
 	virtual bool isend_chrom() const = 0;
 
 	// returns current iterator index
-	virtual size_t iter_index() const = 0;
+	virtual uint64_t iter_index() const = 0;
 
 	// returns current iterator index within the current chromosome (or pair of chromosomes)
-	virtual size_t iter_chrom_index() const = 0;
+	virtual uint64_t iter_chrom_index() const = 0;
 
 	// returns current interval
 	virtual const Interval &cur_interval() const = 0;
