@@ -8,10 +8,13 @@
 #ifndef QUADTREE_H_
 #define QUADTREE_H_
 
+#include <cstdint>
+#include <inttypes.h>
+#include <math.h>
 #include <stdio.h>
+
 #include <algorithm>
 #include <limits>
-#include <math.h>
 #include <queue>
 #include <vector>
 
@@ -40,9 +43,10 @@ struct Rectangle_val : public Rectangle {
 
 	char *debug_str() const {
 		static char str[200];
-		sprintf(str, "(%ld - %ld) (%ld - %ld) %g", x1, x2, y1, y2, (double)v);
+		snprintf(str, sizeof(str), "(%" PRId64 " - %" PRId64 ") (%" PRId64 " - %" PRId64 ") %g", x1, x2, y1, y2, (double)v);
 		return str;
 	}
+
 };
 
 //----------------------------------------- Point_val ---------------------------------------
@@ -60,7 +64,7 @@ struct Point_val : public Point {
 
 	char *debug_str() const {
 		static char str[200];
-		sprintf(str, "(%ld - %ld) %g", x, y, (double)v);
+		snprintf(str, sizeof(str), "(%ld - %ld) %g", x, y, (double)v);
 		return str;
 	}
 };
@@ -279,7 +283,7 @@ public:
 
 			// The order in the heap must give preference to objects before nodes.
 			// Otherwise nearest neighbor query that covers the whole arena will cause all the tree nodes to be added to the heap.
-			bool operator<(const Neighbor &obj) const { return dist > obj.dist || dist == obj.dist && node; }
+			bool operator<(const Neighbor &obj) const { return dist > obj.dist || (dist == obj.dist && node); }
 		};
 
 		Rectangle                 m_query;
@@ -341,7 +345,7 @@ bool StatQuadTree<T, Size>::NNIterator::next()
 		m_neighbors.pop();
 		if (node->is_leaf) {
 			for (uint64_t iobj_ptr = node->leaf.obj_ptr_start_idx; iobj_ptr < node->leaf.obj_ptr_end_idx; ++iobj_ptr) {
-				size_t idx = m_parent->m_obj_ptrs[iobj_ptr];
+				uint64_t idx = m_parent->m_obj_ptrs[iobj_ptr];
 				if (!m_used_objs[idx]) {
 					T &obj = m_parent->m_objs[idx];
 					if (!obj.do_intersect(m_excluded_area)) {
@@ -459,7 +463,7 @@ void StatQuadTree<T, Size>::insert(Node *&node, const Rectangle &intersection, u
 		Rectangle intersection = obj.intersect(quad->arena);
 		if (intersection.is_non_empty_area()) {
 			// after insert() the node pointer might change (m_nodes might get resized)
-			size_t node_idx = node - &*m_nodes.begin();
+			uint64_t node_idx = node - &*m_nodes.begin();
 			insert(quad, intersection, depth + 1, obj, obj_idx);
 			node = &m_nodes[node_idx];
 		}
@@ -510,7 +514,7 @@ void StatQuadTree<T, Size>::insert2leaf(Node *&node, uint64_t obj_idx)
 template <class T, class Size>
 void StatQuadTree<T, Size>::create_quad(Node *&node, int quad, const Rectangle &arena)
 {
-	size_t node_idx = node - &*m_nodes.begin();
+	uint64_t node_idx = node - &*m_nodes.begin();
 	node->node.kid_idx[quad] = m_nodes.size();
 	m_nodes.push_back(Node(arena));
 	node = &m_nodes[node_idx];
@@ -764,34 +768,34 @@ template <class T, class Size>
 void StatQuadTree<T, Size>::debug_print_tree() const
 {
 	debug_print_tree(m_nodes.front(), 0);
-	printf("Objs: %ld\n", m_objs.size());
-	printf("Nodes: %ld\n", m_nodes.size());
-	printf("Obj ptrs: %ld\n", m_obj_ptrs.size());
-	printf("Free chunks: %ld\n", m_obj_ptrs_free_chunks.size());
+	fprintf(stderr, "Objs: %ld\n", m_objs.size());
+	fprintf(stderr, "Nodes: %ld\n", m_nodes.size());
+	fprintf(stderr, "Obj ptrs: %ld\n", m_obj_ptrs.size());
+	fprintf(stderr, "Free chunks: %ld\n", m_obj_ptrs_free_chunks.size());
 }
 
 template <class T, class Size>
 void StatQuadTree<T, Size>::debug_print_tree(const Node &node, unsigned depth) const
 {
-	printf("\n%*sArena: %s\n", depth * 2, "", node.arena.debug_str());
-	printf("%*sIs leaf?: %d\n", (depth + 1) * 2, "", node.is_leaf);
-	printf("%*sArea occupied: %ld\n", (depth + 1) * 2, "", node.stat.occupied_area);
-	printf("%*sAvg: %g\tMin: %g\tMax: %g\n", (depth + 1) * 2, "", node.stat.occupied_area / (double)node.stat.weighted_sum, node.stat.min_val, node.stat.max_val);
+	fprintf(stderr, "\n%*sArena: %s\n", depth * 2, "", node.arena.debug_str());
+	fprintf(stderr, "%*sIs leaf?: %d\n", (depth + 1) * 2, "", node.is_leaf);
+	fprintf(stderr, "%*sArea occupied: %ld\n", (depth + 1) * 2, "", node.stat.occupied_area);
+	fprintf(stderr, "%*sAvg: %g\tMin: %g\tMax: %g\n", (depth + 1) * 2, "", node.stat.occupied_area / (double)node.stat.weighted_sum, node.stat.min_val, node.stat.max_val);
 
 	if (node.is_leaf) {
-		printf("%*sKids: %ld, %ld, %ld\n", (depth + 1) * 2, "", node.leaf.obj_ptr_start_idx, node.leaf.obj_ptr_end_idx, node.leaf.obj_ptr_end_idx - node.leaf.obj_ptr_start_idx);
+		fprintf(stderr, "%*sKids: %ld, %ld, %ld\n", (depth + 1) * 2, "", node.leaf.obj_ptr_start_idx, node.leaf.obj_ptr_end_idx, node.leaf.obj_ptr_end_idx - node.leaf.obj_ptr_start_idx);
 		for (uint64_t i = node.leaf.obj_ptr_start_idx; i < node.leaf.obj_ptr_end_idx; ++i) {
-			printf("%*s%s", (depth + 2) * 2, "", m_objs[m_obj_ptrs[i]].debug_str());
-			printf("\n");
+			fprintf(stderr, "%*s%s", (depth + 2) * 2, "", m_objs[m_obj_ptrs[i]].debug_str());
+			fprintf(stderr, "\n");
 		}
 	} else {
-		printf("%*sNW node\n", (depth + 1) * 2, "");
+		fprintf(stderr, "%*sNW node\n", (depth + 1) * 2, "");
 		debug_print_tree(m_nodes[node.node.kid_idx[NW]], depth + 1);
-		printf("%*sNE node\n", (depth + 1) * 2, "");
+		fprintf(stderr, "%*sNE node\n", (depth + 1) * 2, "");
 		debug_print_tree(m_nodes[node.node.kid_idx[NE]], depth + 1);
-		printf("%*sSE node\n", (depth + 1) * 2, "");
+		fprintf(stderr, "%*sSE node\n", (depth + 1) * 2, "");
 		debug_print_tree(m_nodes[node.node.kid_idx[SE]], depth + 1);
-		printf("%*sSW node\n", (depth + 1) * 2, "");
+		fprintf(stderr, "%*sSW node\n", (depth + 1) * 2, "");
 		debug_print_tree(m_nodes[node.node.kid_idx[SW]], depth + 1);
 	}
 }
