@@ -1,8 +1,8 @@
 #ifndef TRACKEXPRESSIONINTERVALS2DITERATOR_H_
 #define TRACKEXPRESSIONINTERVALS2DITERATOR_H_
 
+#include <cstdint>
 #include "GenomeChromKey.h"
-#include "GenomeTrackRects.h"
 #include "GIntervals2D.h"
 #include "StatQuadTree.h"
 #include "TrackExpression2DIterator.h"
@@ -31,8 +31,6 @@ protected:
 	Rectangles                    m_intersection;
 	vector<uint64_t>              m_intersected_objs_indices;
 	Rectangles::const_iterator    m_iintersection;
-
-	void init(const GenomeChromKey &chromkey, const GIntervals2D &intervals, GIntervalsFetcher2D &scope, const DiagonalBand &band, uint64_t max_data_size);
 };
 
 #endif /* TRACKEXPRESSIONINTERVALS2DITERATOR_H_ */
