@@ -43,8 +43,9 @@ IntervUtils::IntervUtils(SEXP envir)
 	m_kids_intervals1d.clear();
 	m_kids_intervals2d.clear();
 
-	
-	m_allgenome = Rf_findVar(Rf_install("ALLGENOME"), Rf_findVar(Rf_install(".misha"), m_envir));
+    {
+        m_allgenome = find_in_misha(m_envir, "ALLGENOME");
+    }
 
 	if (Rf_isNull(m_allgenome))
 		verror("ALLGENOME variable does not exist");
@@ -80,9 +81,10 @@ IntervUtils::~IntervUtils()
 
 bool IntervUtils::track_exists(const char *track_name)
 {
-	SEXP all_track_names;
+	SEXP all_track_names = R_NilValue;
+	SEXPCleaner all_track_names_cleaner(all_track_names);
 
-	rprotect(all_track_names = Rf_findVar(Rf_install("GTRACKS"), Rf_findVar(Rf_install(".misha"), get_env())));
+	rprotect(all_track_names = find_in_misha(get_env(), "GTRACKS"));
 	if (Rf_isString(all_track_names)) {
 		for (int i = 0; i < Rf_length(all_track_names); ++i) {
 			if (!strcmp(track_name, CHAR(STRING_ELT(all_track_names, i))))
@@ -316,7 +318,9 @@ SEXP IntervUtils::convert_rintervs(SEXP rintervals, GIntervals *intervals, GInte
 		SEXP gintervs;
 		bool interv_found = false;
 
-		rprotect(gintervs = Rf_findVar(Rf_install("GINTERVS"), Rf_findVar(Rf_install(".misha"),m_envir)));
+        {
+            rprotect(gintervs = find_in_misha(m_envir, "GINTERVS"));
+        }
 		if (Rf_isString(gintervs)) {
 			for (int iinterv = 0; iinterv < Rf_length(gintervs); ++iinterv) {
 				const char *interv = CHAR(STRING_ELT(gintervs, iinterv));
@@ -639,9 +643,10 @@ SEXP IntervUtils::convert_chain_intervs(const ChainIntervals &chain_intervs, vec
 	for (ChainIntervals::const_iterator iinterval = chain_intervs.begin(); iinterval != chain_intervs.end(); ++iinterval)
 		tmp_intervals.push_back((GInterval)*iinterval);
 
-	SEXP answer = convert_intervs(&tmp_intervals, ChainInterval::NUM_COLS);
+    SEXP answer = convert_intervs(&tmp_intervals, ChainInterval::NUM_COLS);
 	SEXP src_chroms, src_chroms_idx, src_starts;
-	SEXP col_names = Rf_getAttrib(answer, R_NamesSymbol);
+    SEXP col_names = Rf_getAttrib(answer, R_NamesSymbol);
+    rprotect(col_names);
 	unsigned num_src_chroms = src_id2chrom.size();
 
     rprotect(src_chroms_idx = RSaneAllocVector(INTSXP, chain_intervs.size()));
@@ -656,7 +661,7 @@ SEXP IntervUtils::convert_chain_intervs(const ChainIntervals &chain_intervs, vec
 	for (unsigned id = 0; id < num_src_chroms; ++id)
 		SET_STRING_ELT(src_chroms, id, Rf_mkChar(src_id2chrom[id].c_str()));
 
-	for (int i = 0; i < ChainInterval::NUM_COLS; i++)
+    for (int i = 0; i < ChainInterval::NUM_COLS; i++)
 		SET_STRING_ELT(col_names, i, Rf_mkChar(ChainInterval::COL_NAMES[i]));
 
     Rf_setAttrib(src_chroms_idx, R_LevelsSymbol, src_chroms);
@@ -665,7 +670,8 @@ SEXP IntervUtils::convert_chain_intervs(const ChainIntervals &chain_intervs, vec
     SET_VECTOR_ELT(answer, ChainInterval::CHROM_SRC, src_chroms_idx);
     SET_VECTOR_ELT(answer, ChainInterval::START_SRC, src_starts);
 
-	return answer;
+    runprotect(1); // col_names
+    return answer;
 }
 
 DiagonalBand IntervUtils::convert_band(SEXP rband)
@@ -689,9 +695,9 @@ SEXP IntervUtils::create_data_frame(int numrows, int numcols, SEXP attrs_src)
 {
 	SEXP answer, row_names, col_names;
 
-	rprotect(answer = RSaneAllocVector(VECSXP, numcols));
-    rprotect(col_names = RSaneAllocVector(STRSXP, numcols));
-    rprotect(row_names = RSaneAllocVector(INTSXP, numrows));
+    answer = rprotect_ptr(RSaneAllocVector(VECSXP, numcols));
+    col_names = rprotect_ptr(RSaneAllocVector(STRSXP, numcols));
+    row_names = rprotect_ptr(RSaneAllocVector(INTSXP, numrows));
 
 	for (int i = 0; i < numrows; ++i)
 		INTEGER(row_names)[i] = i + 1;
@@ -703,6 +709,8 @@ SEXP IntervUtils::create_data_frame(int numrows, int numcols, SEXP attrs_src)
     Rf_setAttrib(answer, R_ClassSymbol, Rf_mkString("data.frame"));
     Rf_setAttrib(answer, R_RowNamesSymbol, row_names);
 
+    runprotect(3);
+
 	return answer;
 }
 
@@ -720,8 +728,8 @@ void IntervUtils::define_data_frame_cols(SEXP src, vector<SEXP> &src_cols, SEXP
 		verror("Attempt to copy data frame columns beyond the valid size");
 
 	int numrows = Rf_length(Rf_getAttrib(tgt, R_RowNamesSymbol));
-	SEXP src_colnames = Rf_getAttrib(src, R_NamesSymbol);
-	SEXP tgt_colnames = Rf_getAttrib(tgt, R_NamesSymbol);
+    SEXP src_colnames = rprotect_ptr(Rf_getAttrib(src, R_NamesSymbol));
+    SEXP tgt_colnames = rprotect_ptr(Rf_getAttrib(tgt, R_NamesSymbol));
 
 	if (Rf_isNull(src_colnames) || !Rf_isString(src_colnames))
 		verror("Invalid source data frame for a copy");
@@ -735,7 +743,7 @@ void IntervUtils::define_data_frame_cols(SEXP src, vector<SEXP> &src_cols, SEXP
 		SEXP src_col = VECTOR_ELT(src, col);
 		SEXP tgt_col;
 
-        rprotect(tgt_col = RSaneAllocVector(TYPEOF(src_col), numrows));
+        tgt_col = rprotect_ptr(RSaneAllocVector(TYPEOF(src_col), numrows));
 
 		if (!Rf_isInteger(src_col) && !Rf_isReal(src_col) && !Rf_isLogical(src_col) && !Rf_isString(src_col) && !Rf_isFactor(src_col))
 			verror("Unsupported type found in a data frame: %s", Rf_type2char(TYPEOF(src_col)));
@@ -747,6 +755,7 @@ void IntervUtils::define_data_frame_cols(SEXP src, vector<SEXP> &src_cols, SEXP
 
         SET_VECTOR_ELT(tgt, col + tgt_col_offset, tgt_col);
     }
+    runprotect(2);
 }
 
 void IntervUtils::copy_data_frame_row(const vector<SEXP> &src_cols, int src_row, const vector<SEXP> &tgt_cols, int tgt_row, int tgt_col_offset)
@@ -822,7 +831,7 @@ void IntervUtils::restrict_bins(int64_t maxbins, GIntervals &intervals, unsigned
 bool IntervUtils::get_multitasking() const
 {
 	if (m_multitasking < 0) {
-		SEXP r_multitasking = Rf_GetOption(Rf_install("gmultitasking"), R_NilValue);
+        SEXP r_multitasking = Rf_GetOption1(Rf_install("gmultitasking"));
 
 		if (Rf_isLogical(r_multitasking))
 			m_multitasking = (int)LOGICAL(r_multitasking)[0];
@@ -835,7 +844,7 @@ bool IntervUtils::get_multitasking() const
 uint64_t IntervUtils::get_max_processes() const
 {
 	if (!m_max_processes) {
-		SEXP r_max_processes = Rf_GetOption(Rf_install("gmax.processes"), R_NilValue);
+        SEXP r_max_processes = Rf_GetOption1(Rf_install("gmax.processes"));
 
 		if (Rf_isReal(r_max_processes))
 			m_max_processes = (uint64_t)REAL(r_max_processes)[0];
@@ -852,7 +861,7 @@ uint64_t IntervUtils::get_max_processes() const
 uint64_t IntervUtils::get_max_processes2core() const
 {
 	if (!m_max_processes2core) {
-		SEXP r_max_processes2core = Rf_GetOption(Rf_install("gmax.processes2core"), R_NilValue);
+        SEXP r_max_processes2core = Rf_GetOption1(Rf_install("gmax.processes2core"));
 
 		if (Rf_isReal(r_max_processes2core))
 			m_max_processes2core = (uint64_t)REAL(r_max_processes2core)[0];
@@ -869,7 +878,7 @@ uint64_t IntervUtils::get_max_processes2core() const
 uint64_t IntervUtils::get_min_scope4process() const
 {
 	if (!m_min_scope4process) {
-		SEXP r_min_scope4process = Rf_GetOption(Rf_install("gmin.scope4process"), R_NilValue);
+        SEXP r_min_scope4process = Rf_GetOption1(Rf_install("gmin.scope4process"));
 
 		if (Rf_isReal(r_min_scope4process))
 			m_min_scope4process = (uint64_t)REAL(r_min_scope4process)[0];
@@ -884,7 +893,7 @@ uint64_t IntervUtils::get_min_scope4process() const
 uint64_t IntervUtils::get_max_data_size() const
 {
 	if (!m_max_data_size) {
-		SEXP r_max_data_size = Rf_GetOption(Rf_install("gmax.data.size"), R_NilValue);
+        SEXP r_max_data_size = Rf_GetOption1(Rf_install("gmax.data.size"));
 
 		if (Rf_isReal(r_max_data_size))
 			m_max_data_size = (uint64_t)REAL(r_max_data_size)[0];
@@ -899,7 +908,7 @@ uint64_t IntervUtils::get_max_data_size() const
 uint64_t IntervUtils::get_max_mem_usage() const
 {
 	if (!m_max_mem_usage) {
-		SEXP r_max_mem_usage = Rf_GetOption(Rf_install("gmax.mem.usage"), R_NilValue);
+        SEXP r_max_mem_usage = Rf_GetOption1(Rf_install("gmax.mem.usage"));
 
 		if (Rf_isReal(r_max_mem_usage))
 			m_max_mem_usage = (uint64_t)REAL(r_max_mem_usage)[0] * 1000;
@@ -914,7 +923,7 @@ uint64_t IntervUtils::get_max_mem_usage() const
 uint64_t IntervUtils::get_big_intervals_size() const
 {
 	if (!m_big_intervals_size) {
-		SEXP r_big_intervals_size = Rf_GetOption(Rf_install("gbig.intervals.size"), R_NilValue);
+        SEXP r_big_intervals_size = Rf_GetOption1(Rf_install("gbig.intervals.size"));
 
 		if (Rf_isReal(r_big_intervals_size))
 			m_big_intervals_size = (uint64_t)REAL(r_big_intervals_size)[0];
@@ -930,7 +939,7 @@ uint64_t IntervUtils::get_big_intervals_size() const
 uint64_t IntervUtils::get_quantile_edge_data_size() const
 {
 	if (!m_quantile_edge_data_size) {
-		SEXP r_quantile_edge_data_size = Rf_GetOption(Rf_install("gquantile.edge.data.size"), R_NilValue);
+        SEXP r_quantile_edge_data_size = Rf_GetOption1(Rf_install("gquantile.edge.data.size"));
 
 		if (Rf_isReal(r_quantile_edge_data_size))
 			m_quantile_edge_data_size = (uint64_t)REAL(r_quantile_edge_data_size)[0];
@@ -945,7 +954,7 @@ uint64_t IntervUtils::get_quantile_edge_data_size() const
 uint64_t IntervUtils::get_track_chunk_size() const
 {
 	if (!m_track_chunk_size) {
-		SEXP r_track_chunk_size = Rf_GetOption(Rf_install("gtrack.chunk.size"), R_NilValue);
+        SEXP r_track_chunk_size = Rf_GetOption1(Rf_install("gtrack.chunk.size"));
 
 		if (Rf_isReal(r_track_chunk_size))
 			m_track_chunk_size = (uint64_t)REAL(r_track_chunk_size)[0];
@@ -960,7 +969,7 @@ uint64_t IntervUtils::get_track_chunk_size() const
 uint64_t IntervUtils::get_track_num_chunks() const
 {
 	if (!m_track_num_chunks) {
-		SEXP r_track_num_chunks = Rf_GetOption(Rf_install("gtrack.num.chunks"), R_NilValue);
+        SEXP r_track_num_chunks = Rf_GetOption1(Rf_install("gtrack.num.chunks"));
 
 		if (Rf_isReal(r_track_num_chunks))
 			m_track_num_chunks = (uint64_t)REAL(r_track_num_chunks)[0];
